package com.perforce.hws.testing;

import org.eclipse.jetty.proxy.ProxyServlet;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletRequest;
import java.net.URI;

/**
 * This is a very simple embedded Jetty server that mounts a reverse proxy to a
 * running HWS instance under "/webservices".
 *
 * Ergo, instead of "/api/v16.1"... paths start with "/webservices/api/v16.1/..."
 */
public class ExampleProxy {

    private static final Logger LOGGER =
            LoggerFactory.getLogger(ExampleProxy.class);

    private Server server;

    public ExampleProxy() {
        server = new Server(9001);
    }

    public void start() throws Exception {
        LOGGER.info("initializing proxy");

        ServletContextHandler servletContextHandler = new ServletContextHandler();
        servletContextHandler.setContextPath("/");

        servletContextHandler.addServlet(HwsProxyServlet.class, "/*");

        server.setHandler(servletContextHandler);

        server.start();

        LOGGER.info("proxy ready");
    }

    public void stop() throws Exception {
        server.stop();
    }

    public void join() throws InterruptedException {
        server.join();
    }

    public static void main(final String[] args) throws Exception {
        try {
            ExampleProxy exampleProxy = new ExampleProxy();
            exampleProxy.start();
            exampleProxy.join();

        } catch (Exception e) {
            LOGGER.error("Halting due to error", e);
            throw e;
        }
    }


    public static class HwsProxyServlet extends ProxyServlet {
        @Override
        protected String rewriteTarget(HttpServletRequest clientRequest) {
            // Right now we're forwarding all requests to a dev instance
            // (avoids stupid SSL issues)
            String uri = "http://localhost:9000";

            // Take the current path and append it to the new url
            String path = clientRequest.getRequestURI();
            uri += path.replaceFirst("/webservices", "");

            // Add query params
            String query = clientRequest.getQueryString();
            if (query != null && query.length() > 0) {
                uri += "?" + query;
            }
            return URI.create(uri).normalize().toString();
        }
    }
}
